
/**
 * @cond
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * To improve the quality of the software, users are encouraged to share modifications, enhancements or bug fixes
 * with Infineon Technologies AG (dave@infineon.com).
 ***********************************************************************************************************************
 *
 * Change History
 * --------------
 *
 * 2014-06-16:
 *     - Initial version<br>
 * 2015-08-28:
 *     - Added CLOCK_XMC1_Init conditionally
 *
 * @endcond
 *
 */

/***********************************************************************************************************************
 * HEADER FILES
 **********************************************************************************************************************/
#include "DAVE.h"

/***********************************************************************************************************************
 * API IMPLEMENTATION
 **********************************************************************************************************************/

/*******************************************************************************
 * @brief This function initializes the APPs Init Functions.
 *
 * @param[in]  None
 *
 * @return  DAVE_STATUS_t <BR>
 ******************************************************************************/
DAVE_STATUS_t DAVE_Init(void)
{
  DAVE_STATUS_t init_status;
  
  init_status = DAVE_STATUS_SUCCESS;
     /** @Initialization of APPs Init Functions */
     init_status = (DAVE_STATUS_t)CLOCK_XMC4_Init(&CLOCK_XMC4_0);

  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance TICKS_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&TICKS_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of UART APP instance UART_0 */
	 init_status = (DAVE_STATUS_t)UART_Init(&UART_0); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance TICKS_TIMER_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&TICKS_TIMER_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of ADC_MEASUREMENT APP instance ADC_MEASUREMENT */
	 init_status = (DAVE_STATUS_t)ADC_MEASUREMENT_Init(&ADC_MEASUREMENT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of PIN_INTERRUPT APP instance CAM_SENSOR */
	 init_status = (DAVE_STATUS_t)PIN_INTERRUPT_Init(&CAM_SENSOR); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CAM_LOW_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CAM_LOW_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CAM_HIGH_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CAM_HIGH_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CAM_LOW_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CAM_LOW_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CAM_HIGH_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CAM_HIGH_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance CAM_OUTPUT */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&CAM_OUTPUT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CAM_RPM_TIME */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CAM_RPM_TIME); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of PIN_INTERRUPT APP instance CRANK_SENSOR */
	 init_status = (DAVE_STATUS_t)PIN_INTERRUPT_Init(&CRANK_SENSOR); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CRANK_LOW_TIMER_1 */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CRANK_LOW_TIMER_1); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CRANK_HIGH_TIMER_1 */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CRANK_HIGH_TIMER_1); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CRANK_LOW_TIMER_2 */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CRANK_LOW_TIMER_2); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance CRANK_HIGH_TIMER_2 */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&CRANK_HIGH_TIMER_2); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CRANK_LOW_INTERRUPT1 */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CRANK_LOW_INTERRUPT1); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CRANK_HIGH_INTERRUPT1 */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CRANK_HIGH_INTERRUPT1); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CRANK_LOW_INTERRUPT2 */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CRANK_LOW_INTERRUPT2); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance CRANK_HIGH_INTERRUPT2 */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&CRANK_HIGH_INTERRUPT2); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance CRANK_OUTPUT */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&CRANK_OUTPUT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_1_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_1_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ1_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ1_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ1_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ1_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_1_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_1_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_2_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_2_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_3_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_3_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_4_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_4_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_5_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_5_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_6_SENSE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_6_SENSE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ2_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ2_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ3_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ3_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ4_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ4_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ2_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ2_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ3_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ3_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ5_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ5_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ6_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ6_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_2_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_2_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_3_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_3_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_4_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_4_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_5_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_5_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of DIGITAL_IO APP instance INJ_6_DRIVE */
	 init_status = (DAVE_STATUS_t)DIGITAL_IO_Init(&INJ_6_DRIVE); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of INTERRUPT APP instance INJ4_INTERRUPT */
	 init_status = (DAVE_STATUS_t)INTERRUPT_Init(&INJ4_INTERRUPT); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ5_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ5_TIMER); 
   } 
  if (init_status == DAVE_STATUS_SUCCESS)
  {
	 /**  Initialization of TIMER APP instance INJ6_TIMER */
	 init_status = (DAVE_STATUS_t)TIMER_Init(&INJ6_TIMER); 
   }  
  return init_status;
} /**  End of function DAVE_Init */

