/*
 * main.c
 *
 *  Created on: 2017 Oct 13 16:34:53
 *  Author: Rory Thompson
 */




#include <DAVE.h>                 //Declarations from DAVE Code Generation (includes SFR declaration)
#include <stdint.h>
#include "src\global\global.h"
#include "src\adc\read_analogues.h"
#include "src\calc_fuel_map\calc_fuel_map.h"
#include "src\cam_control\cam_control.h"
#include "src\crank_control\crank_control.h"
#include "src\calc_timing_delay\calc_timing_delay.h"
#include "src\injector_control\injector_control.h"
#include "src\misc_functions\misc_functions.h"

int8_t SerialData[40];
int32_t camDelayTime = 10;
int32_t crankDelayAngle = 0;
int32_t crankDelayTime = 10;
int32_t ticks = 0;
int32_t lastTicks = 0;
int32_t lastTicks2 = 0;
int32_t extraFuel = 0; //result from fuel map
int32_t extraFuelTime = 0; //result from fuel map scaled to suit timers














/**

 * @brief main() - Application entry point
 *
 * <b>Details of function</b><br>
 * This routine is the application entry point. It is invoked by the device startup code. It is responsible for
 * invoking the APP initialization dispatcher routine - DAVE_Init() and hosting the place-holder for user application
 * code.
 */

int main(void)
{
  DAVE_STATUS_t status;

  status = DAVE_Init();           /* Initialization of DAVE APPs  */

  if(status != DAVE_STATUS_SUCCESS)
  {
    /* Placeholder for error handler code. The while loop below can be replaced with an user error handler. */
    XMC_DEBUG("DAVE APPs initialization failed\n");

    while(1U)
    {

    }
  }

  /* Placeholder for user application code. The while loop below can be replaced with user application code. */
  while(1U)
  {

if((ticks - lastTicks) > 5)
{
    ADC_MEASUREMENT_StartConversion(&ADC_MEASUREMENT);
    crankDelayAngle = getFromTimingMap(RPM, manifoldPress); //camDelay(RPM, manifoldPress);
    camDelayTime = camDelay(crankDelayAngle, RPM);
    crankDelayTime = crankDelay(camDelayTime);
    extraFuel = getFromFuelMap(RPM, manifoldPress);
    extraFuelTime = fuelDelay(extraFuel);
    lastTicks = ticks;
}

if((ticks - lastTicks2) > 10)
{

	mainMaxDuy = maxDutyCalc();
	sprintf(SerialData, "%i,%i,%i,%i,%i,%i,%i\r\n",RPM, manifoldPress , wideBandLambda, extraFuel, crankDelayAngle, mainMaxDuy,lambdaTemp);
    UART_Transmit(&UART_0, SerialData, strlen(SerialData));
    lastTicks2 = ticks;
}


if((ADCDataAvailable == 1))
{
	Adc_Calc();
	ADCDataAvailable = 1;
}



if(!DIGITAL_IO_GetInput(&INJ_1_SENSE) && inj1High == 0 ) //input goes low when injector opens
{
	if(injector1Status == 1)
	{
		injector1MaxDuy = 1;
	}
	else
	{
		injector1MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_1_DRIVE);
	  inj1High = 1;
	  injector1Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_1_SENSE) && inj1High == 1 ) //input goes low when injector closes
{
	  injector1Status = 0;
	  TIMER_Stop(&INJ1_TIMER);
	  TIMER_SetTimeInterval(&INJ1_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ1_TIMER);
	  TIMER_Start(&INJ1_TIMER);
	  inj1High = 0;

}


if(!DIGITAL_IO_GetInput(&INJ_2_SENSE) && inj2High == 0 ) //input goes low when injector opens
{
	if(injector2Status == 1)
	{
		injector2MaxDuy = 1;
	}
	else
	{
		injector2MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_2_DRIVE);
	  inj2High = 1;
	  injector2Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_2_SENSE) && inj2High == 1 ) //input goes low when injector closes
{
	  injector2Status = 0;
	  TIMER_Stop(&INJ2_TIMER);
	  TIMER_SetTimeInterval(&INJ2_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ2_TIMER);
	  TIMER_Start(&INJ2_TIMER);
	  inj2High = 0;

}

if(!DIGITAL_IO_GetInput(&INJ_3_SENSE) && inj3High == 0 ) //input goes low when injector opens
{
	if(injector3Status == 1)
	{
		injector3MaxDuy = 1;
	}
	else
	{
		injector3MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_3_DRIVE);
	  inj3High = 1;
	  injector3Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_3_SENSE) && inj3High == 1 ) //input goes low when injector closes
{
	  injector3Status = 0;
	  TIMER_Stop(&INJ3_TIMER);
	  TIMER_SetTimeInterval(&INJ3_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ3_TIMER);
	  TIMER_Start(&INJ3_TIMER);
	  inj3High = 0;

}

if(!DIGITAL_IO_GetInput(&INJ_4_SENSE) && inj4High == 0 ) //input goes low when injector opens
{
	if(injector4Status == 1)
	{
		injector4MaxDuy = 1;
	}
	else
	{
		injector4MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_4_DRIVE);
	  inj4High = 1;
	  injector4Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_4_SENSE) && inj4High == 1 ) //input goes low when injector closes
{
	  injector4Status = 0;
	  TIMER_Stop(&INJ4_TIMER);
	  TIMER_SetTimeInterval(&INJ4_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ4_TIMER);
	  TIMER_Start(&INJ4_TIMER);
	  inj4High = 0;

}

if(!DIGITAL_IO_GetInput(&INJ_5_SENSE) && inj5High == 0 ) //input goes low when injector opens
{
	if(injector5Status == 1)
	{
		injector5MaxDuy = 1;
	}
	else
	{
		injector5MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_5_DRIVE);
	  inj5High = 1;
	  injector5Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_5_SENSE) && inj5High == 1 ) //input goes low when injector closes
{
	  injector5Status = 0;
	  TIMER_Stop(&INJ5_TIMER);
	  TIMER_SetTimeInterval(&INJ5_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ5_TIMER);
	  TIMER_Start(&INJ5_TIMER);
	  inj5High = 0;

}

if(!DIGITAL_IO_GetInput(&INJ_6_SENSE) && inj6High == 0 ) //input goes low when injector opens
{
	if(injector6Status == 1)
	{
		injector6MaxDuy = 1;
	}
	else
	{
		injector6MaxDuy = 0;
	}

	  DIGITAL_IO_SetOutputHigh(&INJ_6_DRIVE);
	  inj6High = 1;
	  injector6Status = 1;
}

if(DIGITAL_IO_GetInput(&INJ_6_SENSE) && inj6High == 1 ) //input goes low when injector closes
{
	  injector6Status = 0;
	  TIMER_Stop(&INJ6_TIMER);
	  TIMER_SetTimeInterval(&INJ6_TIMER, extraFuelTime);
	  TIMER_Clear(&INJ6_TIMER);
	  TIMER_Start(&INJ6_TIMER);
	  inj6High = 0;

}


  }
}



void ticksTimerInterrupt()  // called every 10ms
{
ticks ++;
}



