/*
 *
 *
 *Calculates result from fuel map below
 *returns interpolated result from table x 100 to accommodate for two decimal paces
 *
 */

#include "CALC_FUEL_MAP.h"


uint16_t fuelMap [18][17] = {                // this is extra injector time in ms x 10, for example 2.8ms will be 28 in table
  {800, 80,  80,    80,   80,   80,  80,   80,   80,   80,  80,   80,   80,   80,   80,    80,   80},
  {750, 75,  75,    75,   75,   75,  75,   75,   75,   90,  75,   75,   75,   75,   75,    75,   75},
  {700, 70,  70,    70,   70,   70,  70,   70,   70,   85,  70,   70,   70,   70,   70,    70,   70},
  {650, 65,  65,    65,   65,   60,  65,   65,   65,   80,  65,   65,   65,   65,   65,    65,   65},
  {600, 60,  60,    60,   60,   55,  62,   60,   80,   75,  65,   60,   60,   60,   60,    60,   60},
  {550, 55,  55,    55,   55,   50,  60,   55,   75,   70,  60,   55,   55,   55,   55,    55,   55},
  {500, 50,  50,    50,   50,   45,  55,   60,   70,   60,  55,   50,   50,   50,   50,    50,   50},
  {450, 45,  45,    45,   45,   40,  50,   55,   65,   60,  50,   45,   45,   45,   45,    45,   45},
  {400, 40,  40,    40,   40,   35,  45,   50,   60,   55,  45,   40,   40,   40,   40,    40,   40},
  {350, 35,  35,    35,   35,   30,  40,   45,   55,   45,  40,   35,   35,   35,   35,    35,   35},
  {300, 30,  30,    30,   30,   25,  35,   40,   50,   40,  35,   30,   30,   30,   30,    30,   30},
  {250, 25,  25,    25,   25,   20,  20,   25,   35,   35,  30,   25,   25,   25,   25,    25,   25},
  {200, 20,  20,    20,   20,   15,  15,   20,   30,   30,  25,   20,   20,   20,   20,    20,   20},
  {150, 15,  15,    15,   15,   10,  10,   15,   20,   20,  15,   15,   15,   15,   15,    15,   15},
  {100, 10,  10,    10,   10,   10,  10,   10,   10,   10,  10,   10,   10,   10,   10,    10,   10},
  {50,  05,  05,    05,   05,   05,  05,   05,   05,   05,  05,   05,   05,   05,   05,    05,   05},
  {0,   0,    0,     0,   0,    0,    0,    0,    0,    0,   0,    0,    0,    0,    0,     0,    0},
  {0,   0, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500, 8000},
};


// Get from fuel map COMPLEX. x = RPM, y = boost pressure. works on 18x16 fuel map. Determines interpolated value
int16_t getFromFuelMap(int x, int y) {
  /// this calculates y position in array - RPM   //example input 523
  int32_t xPositionLow;   //x row bottom position in table
  int32_t xPositionHigh;   //x row top position in table
  int32_t yPositionLow;   //y column left position in table
  int32_t yPositionHigh;  //y column right position in table
  int32_t A;  //bottom left
  int32_t B;  //bottom right
  int32_t C;  //top left
  int32_t D;  //top right
  float ac; //position between A and C
  float bd; //position between B and D
  int32_t xValLow; // low value of x axis
  int32_t xValHigh;  // high value of x axis
  int32_t yValLow;   // low value of y axis
  int32_t yValHigh;  // high value of 7 axis
  int32_t yPercent;
  int32_t xPercent;
  float preResult; //used as temp value
  int32_t result; // interpolated output. point between ac and bd



  // limit inputs to maximum value from table

  if (x >= 8000) {                         //constrain to max of table
     x= 8000;
   }

  if (y >= 800) {                         //constrain to max of table
     y = 800;
   }

  // this calculates X position in array (RPM)

  if (x >= 7500) {                         //constrain to max of table
    xPositionLow = 15;
  }
  else {
    if (x <= 1000) {                       //constrain to min of table
      xPositionLow = 1;
    }
    else {
      for (int16_t tempVal1 = 15; tempVal1 >= 0 ; tempVal1 --) {  // runs through the table and determines the bottom cell from the pair that the boost pressure lies within

        int16_t tempVal2 = fuelMap [17] [tempVal1];  // [Y position] [X position] read from left column
        if (tempVal2 <= x) {
          xPositionLow = tempVal1;
          break;
        }
      }
    }
  }


  // this calculates Y position in array (boost pressure)

  if (y <= 50) {                            //if boost pressure is negaive go to 0 in table
    yPositionLow = 16;
  }
  else {

    if (y >= 750) {                       //if boost pressure is negaive go to 750 in table
      yPositionLow = 1;
    }
    else {

      for (int16_t tempVal1 = 17; tempVal1 >= 0 ; tempVal1 --) {  // runs through the table and determines the left cell from the pair that the boost pressure lies within

        int tempVal2 = fuelMap [tempVal1] [0];  // [Y position] [X position] read from bottom column

        if (tempVal2 >= y) {
          yPositionLow = tempVal1;
          yPositionLow ++;
          break;
        }
      }
    }
  }

  // get corners of square
  xPositionHigh = xPositionLow + 1;
  yPositionHigh = yPositionLow - 1;
  A = fuelMap [yPositionLow] [xPositionLow];  // [Y position] [X position] read from map
  B = fuelMap [yPositionLow] [xPositionHigh];  // [Y position] [X position] read from map
  C = fuelMap [yPositionHigh] [xPositionLow];  // [Y position] [X position] read from map
  D = fuelMap [yPositionHigh] [xPositionHigh];  // [Y position] [X position] read from map

  //get x and y axis values for corners of square
  yValHigh = fuelMap [yPositionHigh] [0];
  yValLow = fuelMap [yPositionLow] [0];
  xValHigh = fuelMap [17] [xPositionHigh];
  xValLow = fuelMap [17] [xPositionLow];

  // calculate position of actual readings between axis values, gives percentage. example 45 = 45%

  if (y < 0) {         // contstrain map to 0 to stop calc breaking at manifold vaccum
    y = 0;
  }

  yPercent = ((y - yValLow) * 100);
  yPercent = yPercent / (yValHigh - yValLow); //calc done in two steps to avoid strange buffer overflows

  xPercent = ((x - xValLow) * 100);
  xPercent = xPercent / (xValHigh - xValLow); //calc done in two steps to avoid strange buffer overflows

  // calculate mid points on y columns

  ac = (C - A);
  ac = ((ac/100) * yPercent) + A;

  bd = (D - B);
  bd = (((float)bd / 100) * yPercent) + B;

  // calculate mid point between ac and bd
  preResult = bd - ac;
  result = ((((float)preResult / 100 * xPercent) + ac) * 100);
  return result;
}
